import Command from "./command";

import db from "../things/db";
import bot from "../things/bot";

import {
  Constants,
  InteractionDataOptionsString
} from "@projectdysnomia/dysnomia";
import * as chrono from "chrono-node";
import logger from "../things/logger";

import {
  Optional,
  unravelOption,
  unravelOptionalOption
} from "../utils/options";
import { getCurrentOffset, getTimeOffsetFor } from "../utils/time";

function parse(str: string, offset: number) {
  const when = chrono.parse(
    str,
    {
      instant: new Date(),
      timezone: offset
    },
    {
      forwardDate: true
    }
  );

  if (when.length <= 0) return null;
  return when[0].start.date();
}

const remind: Command = {
  name: "remind",
  description: "Reminds you to do something.",
  options: [
    {
      name: "when",
      description: "When to remind you.",
      required: true,
      type: Constants.ApplicationCommandOptionTypes.STRING
    },
    {
      name: "what",
      description: "What to remind you about.",
      type: Constants.ApplicationCommandOptionTypes.STRING
    }
  ],
  command: async (interaction) => {
    // i love abusing types
    const options = interaction.data.options ?? [];
    const whenOption = unravelOption<InteractionDataOptionsString>(
      "when",
      options
    );
    const whatOption = unravelOptionalOption<
      Optional<InteractionDataOptionsString>
    >("what", options);

    const user = (interaction.member || interaction.user)!.id;
    const now = new Date();

    const offset = (await getTimeOffsetFor(user)) ?? 0;
    const when = parse(whenOption.value, offset);

    if (when == null) {
      let msg =
        ":warning: I didn't understand that time. Try something like `3h` or `2 days`.";
      if (offset != 0) {
        msg += "\nNote: This may be a bug due to your set time zone.";
      }
      await interaction.createMessage({
        content: msg,
        flags: Constants.MessageFlags.EPHEMERAL
      });
      return;
    }

    await db.reminder.create({
      data: {
        user,
        createdAt: now,
        sendAt: when,
        message: whatOption?.value
      }
    });

    await interaction.createMessage({
      content: `Reminder set <t:${Math.floor(when.getTime() / 1000)}:R>.`,
      flags: Constants.MessageFlags.EPHEMERAL
    });
  }
};

async function checkDB() {
  const reminders = await db.reminder.findMany({
    where: {
      sendAt: {
        lte: new Date()
      }
    }
  });

  for (const reminder of reminders) {
    try {
      let user = await bot.users.find((x) => x.id === reminder.user);
      if (!user) user = await bot.getRESTUser(reminder.user);
      if (!user) throw new Error("User not found");

      const dmChannel = await user.getDMChannel();

      await dmChannel.createMessage(
        `You asked me to remind you about ${
          reminder.message ?? "something"
        } <t:${Math.floor(reminder.createdAt.getTime() / 1000)}:R>.`
      );
    } catch (err) {
      logger.error(
        { err, id: reminder.user },
        "Couldn't send reminder to user"
      );
    }

    await db.reminder.delete({
      where: {
        id: reminder.id
      }
    });
  }
}

setInterval(checkDB, 1000 * 5);

export default remind;
