import { findTimeZone, getUTCOffset } from "timezone-support";
import db from "../things/db";

export function getCurrentOffset() {
  // negative offset = west
  return -new Date().getTimezoneOffset();
}

export async function updateTimeFor(
  user: string,
  tzName: string | null
): Promise<boolean> {
  if (tzName == null) {
    await db.timezone.upsert({
      where: { user },
      create: { user, tz: null },
      update: { tz: null }
    });

    return true;
  }

  try {
    const tz = findTimeZone(tzName).name;

    await db.timezone.upsert({
      where: { user },
      create: { user, tz },
      update: { tz }
    });

    return true;
  } catch (err) {
    return false;
  }
}

export async function getTimeNameFor(user: string): Promise<string | null> {
  try {
    const lookup = await db.timezone.findUnique({
      where: {
        user
      }
    });

    const tzName = lookup?.tz;
    if (tzName == null) return null;
    const tz = findTimeZone(tzName);

    return tz.name;
  } catch (err) {
    return null;
  }
}

export async function getTimeOffsetFor(user: string): Promise<number | null> {
  try {
    const lookup = await db.timezone.findUnique({
      where: {
        user
      }
    });

    const tzName = lookup?.tz;
    if (tzName == null) return null;

    const tz = findTimeZone(tzName);
    const offsetObj = getUTCOffset(new Date(), tz);
    return -offsetObj.offset;
  } catch (err) {
    return 0;
  }
}

export async function getTimeFor(user: string): Promise<Date | null> {
  try {
    const tzName = await getTimeNameFor(user);
    if (tzName == null) return null;

    const now = new Date();
    const tz = findTimeZone(tzName);

    const offsetMinutes = getUTCOffset(now, tz).offset;
    const offsetMs = offsetMinutes * 60 * 1000;
    const nowOffset = getCurrentOffset() * 60 * 1000;

    return new Date(now.getTime() - offsetMs - nowOffset);
  } catch (err) {
    return null;
  }
}
